/*! ********************************************
 * Tone.hpp
 * Copyright 2019 masahiro takamoto (MotChan)
 * Distributed under the MIT license. 
 * see -> sora/license.txt
 ******************************************** */
#ifndef INCLUDE_Tone_h
#define INCLUDE_Tone_h

#ifndef INCLUDE_GrayBox_Types_h
#include "Types.h"
#endif
namespace sora {
namespace graybox {
namespace synth {
/***********************************************
	Class	SamplerPreset
************************************************/
class SamplerPreset
{
public:
	//int32* parameterPointer[16];
	char toneName[17]={"                "};	// ASCII 16Chars 
	int32 smplNum = 0;
	int32 pan = 64;
	int32 octv = 8;     // 0 - 16     // center 8, octave = 1
	int32 tune = 8192;  // 0 - 16383  // center 8192, octave = 8192
	int32 tl = 0;       // 0 - 127
	int32 ar = 127;     // 0 - 127
	int32 dr = 0;       // 0 - 127
	int32 sl = 127;     // 0 - 127
	int32 rr = 120;     // 0 - 127
	
	void copy(const SamplerPreset& preset);
	void reset();
};
/***********************************************
	Class	TonePresetWT
************************************************/
class TonePresetWT
{
public:
	char toneName[17]={"                "};	// ASCII 16Chars 
	int32 key = 64;        // 0 - 127     *Percussion
	int32 pan = 64;        // 0 - 127     *Percussion
	int32 lfoFreq = 2048;  // 0 - 16383   LFO - Phase Update Timer Parameter
	int32 strc = 0;        // 0 - 3
	int32 tune = 8192;     // 0 - 16383   center 8192, octave = 8192
	int32 lms = 0;         // 0 - 15      LFO - Modulation Depth Senstivity
	int32 wave = 0;        // 0 - 127
	int32 tl = 0;          // 0 - 127
	int32 ar = 127;        // 0 - 127
	int32 dr = 100;        // 0 - 127
	int32 sl = 0;          // 0 - 127
	int32 rr = 120;        // 0 - 127
	int32 kf = 0;          // 0 - 3       Key Follow Sensitivity         *Instrument 
	int32 kfStart = 48;    // 0 - 127     Key Number - Start Key Follow  *Instrument
	
	void copy(const TonePresetWT& preset);
	void reset();
};
/***********************************************
	Class	TonePresetPCM
************************************************/
class TonePresetPCM
{
public:
	char toneName[17]={"                "};	// ASCII 16Chars 
	int32 lfoFreq = 2048;  // 0 - 16383   LFO - Phase Update Timer Parameter
	int32 strc = 0;        // 0 - 3
	int32 tune = 8192;     // 0 - 16383   center 8192, octave = 8192
	int32 lms = 0;         // 0 - 15      LFO - Modulation Depth Senstivity
	int32 smplNum = 0;     // 0 - 127
	int32 tl = 0;          // 0 - 127
	int32 ar = 126;        // 0 - 127
	int32 dr = 100;        // 0 - 127
	int32 sl = 0;          // 0 - 127
	int32 rr = 120;        // 0 - 127
	int32 kf = 0;          // 0 - 3       Key Follow Sensitivity         *Instrument 
	int32 kfStart = 48;    // 0 - 127     Key Number - Start Key Follow  *Instrument
	
	void copy(const TonePresetPCM& preset);
	void reset();
};
/***********************************************
	Class	TonePresetFM
************************************************/
class TonePresetFM
{
public:
	char toneName[17]={"                "};	// ASCII 16Chars 
	int32 key = 64;        // 0 - 127     *Percussion
	int32 pan = 64;        // 0 - 127     *Percussion
	int32 lfoWave = 0;     // 0 - 3
	int32 lfoFreq = 2048;  // 0 - 16383   LFO - Phase Update Timer Parameter
	int32 fl = 127;        // 0 - 127    // Feed Back Level
	int32 strc = 0;        // 0 - 7	  // strcture
	// operator 0
	//int32 velCurve_0 = 0;
	int32 tlVelCtrl_0 = 0;// 0 - 7      Total Level Contorol - Velocity
	int32 mul_0 = 1;      // 0 - 7      Multiple       [0] = 1/2 Cycle, [1] = 1/1, [2] = 2/1, [3] = 3/1...
	int32 tune_0 = 8192;  // 0 - 16383  center 8192, octave = 8192
	int32 lms_0 = 0;      // 0 - 15      LFO - Modulation Depth Senstivity
	int32 wave_0 = 0;     // 0 - 7
	int32 tl_0 = 0;       // 0 - 127
	int32 ar_0 = 124;     // 0 - 127
	int32 dr_0 = 100;     // 0 - 127
	int32 sl_0 = 0;       // 0 - 127
	int32 rr_0 = 120;     // 0 - 127
	int32 kf_0 = 0;       // 0 - 3      Key Follow Sensitivity         *Instrument 
	int32 kfStart_0 = 48; // 0 - 127    Key Number - Start Key Follow  *Instrument
	// operator 1
	//int32 velCurve_1 = 0;
	int32 tlVelCtrl_1 = 0;// 0 - 7
	int32 mul_1 = 1;      // 0 - 7
	int32 tune_1 = 8192;  // 0 - 16383
	int32 lms_1 = 0;     // 0 - 15      LFO - Modulation Depth Senstivity
	int32 wave_1 = 0;     // 0 - 7
	int32 tl_1 = 0;       // 0 - 127
	int32 ar_1 = 124;     // 0 - 127
	int32 dr_1 = 100;     // 0 - 127
	int32 sl_1 = 0;       // 0 - 127
	int32 rr_1 = 120;     // 0 - 127
	int32 kf_1 = 0;       // 0 - 3
	int32 kfStart_1 = 48; // 0 - 127
	
	void copy(const TonePresetFM& preset);
	void reset();
};
}}}
#endif